#include <cstdio> // for printf
#include <windows.h> // for QueryPerformanceCounter/QueryPerformanceFrequency
#include "generator.h"

const int NUM_PROBLEMS = 10000;
// define this as 1 to print all solutions to the console
#define SHOW_SOLUTIONS 0

// solvers included by header
#include "example.h"

// array of solver programs to be tested
struct SolverTest
{
	const char * name;
	void (*solver)( const char * problem, char * solution );
};

static const SolverTest tests[] =
{
	{ "Example::bad_answer", Example::bad_answer },
};

const int NUM_TESTS = sizeof(tests)/sizeof(tests[0]);

// Win32 timer

LARGE_INTEGER start_time;

void start_timer()
{
	::QueryPerformanceCounter(&start_time);
}

double end_timer()
{
	LARGE_INTEGER end_time;
	::QueryPerformanceCounter(&end_time);

	LARGE_INTEGER freq;
	::QueryPerformanceFrequency(&freq);

	LONGLONG diff = end_time.QuadPart - start_time.QuadPart;
	
	return double(diff) / double(freq.QuadPart);
}

// main test

int main(int argc, char ** argv)
{
	char * problems = new char[ 81 * NUM_PROBLEMS ];
	char * solutions = new char[ 81 * NUM_PROBLEMS ];

	start_timer();
	unsigned int seed = start_time.LowPart; // generate a seed from the timer
	Generator::seed_generator(seed);
	::printf("Generating %d problems from seed: %u\n", NUM_PROBLEMS, seed );

	for ( int i=0; i < NUM_PROBLEMS; ++i )
	{
		Generator::generate( problems + (81 * i) );
	}
	::printf("Generation took %12.6f seconds.\n", end_timer() );

	::printf("Performing %d tests...\n",NUM_TESTS);
	for ( int t=0; t < NUM_TESTS; ++t )
	{
		// wipe previous solutions
		for ( int i=0; i < (81 * NUM_PROBLEMS); ++i )
		{
			solutions[i] = 0;
		}

		char * problem = problems;
		char * solution = solutions;

		// timed test of all problems

		start_timer();

		for ( int p=0; p < NUM_PROBLEMS; ++p )
		{
			tests[t].solver( problem, solution );
			problem += 81;
			solution += 81;
		}

		double duration = end_timer();

		// verify solutions
		problem = problems;
		solution = solutions;
		int solved = 0;
		for ( int p=0; p < NUM_PROBLEMS; ++p )
		{
			bool solved_this = Generator::solved( problem, solution );
			if ( solved_this ) ++solved;

#if SHOW_SOLUTIONS
			::printf("\n");
			if ( solved_this )
				::printf("SOLVED %d\n",p);
			else
				::printf("FAILED %d\n",p);
			Generator::print2(problem, solution);
#endif

			problem += 81;
			solution += 81;
		}

		// print stats
		::printf("%4d: %25s solved %7d/%7d in %12.6f seconds.\n",
			t, tests[t].name, solved, NUM_PROBLEMS, duration );
	}
	::printf("Done.\n");

	return 0;
}

// END OF FILE
